/************************************************************************
* "Epsilon" is a registered trademark licensed to Lugaru Software, Ltd. *
*	"EEL" and "Lugaru" are trademarks of Lugaru Software, Ltd.	*
*									*
*  Copyright (C) 1985, 1992 Lugaru Software Ltd.  All rights reserved.	*
*									*
* Limited permission is hereby granted to reproduce and modify this	*
* copyrighted material provided that the resulting code is used only in	*
* conjunction with Lugaru products and that this notice is retained in	*
* any such reproduction or modification.				*
************************************************************************/

/* Support for Ultravision screen size selection from Epsilon. */

/* Note: Big screen sizes require Epsilon to allocate more memory.
If a concurrent process is running the first time you switch to a a
big screen size, Epsilon may run out of memory.  It's better to exit
any concurrent process before switching to a big screen size for the
first time. */

#include "eel.h"
#include "lowlevel.h"

#if EELVERSION < 65
#define user
#endif

user char ultravision_modes = 1;	/* set to 0 to disable this */

	/* for convenience, pass around "size", not lines&cols */
#define MAKE_SIZE(cols, lines)	((lines) * 1000 + (cols))
#define GET_COLS(size)		((size) % 1000)
#define GET_LINES(size)		((size) / 1000)

int _inside_size;	/* screen size while in epsilon */
int _outside_size;	/* size during push or at exit */
short _last_video_mode = -1;		/* last video mode we set */

char *uvision_match();

/* The next two functions replace the standard ones in Epsilon. */

char *vid_match(s, start)	/* completion helper: gives funcs/vars */
char *s;			/* starting w/ "video-mode-" and (if func) */
{				/* return nonzero when called with arg=2 */
	char name[30];
	int i;

	if (have_uvision())
		return uvision_match(s, start);
	sprintf(name, "video-mode-%s", s);
	for (; i = name_match(name, start & STARTMATCH); start = 0)
		if (name_type(i) == NT_VAR || (*(int (*)())i)(2))
			return name_name(i) + 11;
	return 0;
}

change_video(size, active)	/* set to size code, return 1 if ok */
{
	char name[20];
	int i, ret = 0, lines, cols;

	lines = GET_LINES(size);
	cols = GET_COLS(size);
#if EELVERSION >= 65
	save_var mouse_auto_on = 0, mouse_display = 0;
#endif
	if (have_uvision())
		return uvision_set(cols, lines, active);
	sprintf(name, "video-mode-%dx%d", cols, lines);
	i = find_index(name);
	switch (name_type(i)) {
		case NT_SUBR: case NT_COMMAND:
#if EELVERSION >= 65
		case NT_AUTOSUBR: case NT_AUTOLOAD:
#endif
			ret = (*(int (*)())i)(active);
			break;
#ifdef MSDOS
		case NT_VAR:		/* use dos mode # */
			if (active == 1)
				do_video_var(*(char *)varptr(i), lines, cols);
			else if (!active)	/* go to normal mode */
				do_video_var(screen_mode, lines, cols);
			ret = 1;
			break;
#endif
	}
	if (active == 1 && ret) {
		resize_screen(lines, cols);
		when_resizing();
	}
	return ret;
}

have_uvision()		/* is ultravision 1.20 or later installed? */
{
	if (!ultravision_modes)	/* pretend it's not there */
		return 0;
	m_regs.w.cx = 0;	/* see if ultravision is present, get vers */
	m_regs.w.si = 0;	/* early versions don't set si */
	vid_int(0xcc00);	/* check for ultravision */
	return ((m_regs.w.cx & 0xffff) == 0xabcd && m_regs.w.si >= 0x114);
}

struct uv_info {	/* structure for load_uv_info and get_uv_info */
	short table_seg, table_off;	/* memory location of uv table */
	short mode, lines, cols;	/* info on indicated mode */
} uv_info;

load_uv_info()	/* call this subr before calling get_uv_info() below */
{
	vid_int(0xcd0f);	/* get configuration */
	uv_info.table_seg = m_regs.w.ds;
	uv_info.table_off = m_regs.w.di;
}

get_uv_info(index)	/* fill in structure for index'th mode */
{			/* index with mode of -1 is last valid one */
	int n, off;

	n = index * 8;
	uv_info.mode = uv_table_word(n);
	off = uv_table_word(n + 6);
	uv_info.cols = uv_byte(off);
	uv_info.lines = uv_byte(off + 1) + 1;
}

uv_table_word(offset)	/* return a 16-bit word from uv table */
{
	return uv_byte(uv_info.table_off + offset) +
		256 * uv_byte(uv_info.table_off + offset + 1);
}

uv_byte(offset)		/* return a byte from uv data segment */
{
	return peek(uv_info.table_seg, offset);
}

int _uv_match_pos;	/* index of next match to try */
char _uv_match[20];	/* current match */

char *uvision_match(s, start)	/* completion helper func */
char *s;			/* return next or first match for s */
{
	if (start & STARTMATCH)
		_uv_match_pos = 0;
	load_uv_info();
	for (;;) {
		get_uv_info(_uv_match_pos);
		if (uv_info.mode == -1)
			break;
		_uv_match_pos++;
		sprintf(_uv_match, "%dx%d", uv_info.cols, uv_info.lines);
		_uv_match[strlen(s)] = 0;
		if (strcmp(_uv_match, s))	/* doesn't match */
			continue;
		sprintf(_uv_match, "%dx%d", uv_info.cols, uv_info.lines);
		return _uv_match;
	}
	return 0;
}

uvision_set(cols, lines, active)	/* set or check if mode is avail */
{
	int i = 0;

	if (!active)		/* leaving this mode, so do nothing */
		return 1;
	load_uv_info();
	do {
		get_uv_info(i++);	/* see if this mode's right */
		if (cols == uv_info.cols && lines == uv_info.lines) {
			if (active == 1) {	/* really do it */
				vid_int(0xcd00 + (uv_info.mode & 0xff));
				if (m_regs.w.ax == 0xcdcd)
					return 0;	/* bad mode */
				vid_clear_screen(lines, cols);
				term_position(0, 0);
				leave_blank = 1;
				resize_screen(lines, cols);
				when_resizing();
			}
			return 1;
		}
	} while (uv_info.mode != -1); /* try next mode */
	return 0;		/* ran out of modes to try */
}
